//******************************************************************************
//
// SysEx2CS / SEKuchiPakuCtrl
//
// ςNX
//
// Copyright (C) 2023 WADA Masashi. All Rights Reserved.
//
//******************************************************************************

#include "framework.h"
#include "SEKuchiPakuCtrl.h"


//******************************************************************************
// RXgN^
//******************************************************************************
SEKuchiPakuCtrl::SEKuchiPakuCtrl(void)
{
	m_hWnd = NULL;
	m_KuchiPakuCommadIndex = 0;
	m_OutputCharSize = 0;
	m_SpeedRatio = 1.0f;
}

//******************************************************************************
// fXgN^
//******************************************************************************
SEKuchiPakuCtrl::~SEKuchiPakuCtrl(void)
{
}

//******************************************************************************
// 
//******************************************************************************
int SEKuchiPakuCtrl::Initialize(
		HWND hWnd,
		SESysExOutCtrl* pSysExOutCtrl
	)
{
	int result = 0;

	m_hWnd = hWnd;
	m_pSysExOutCtrl = pSysExOutCtrl;

//EXIT:;
	return result;
}

//******************************************************************************
// ςJn
//******************************************************************************
int SEKuchiPakuCtrl::StartKuchiPaku(
		const char* pStr,
		float speedRatio
	)
{
	int result = 0;
	SEKuchiPakuCommandList::iterator itr;
	SEKuchiPakuCommand command;

	m_KuchiPakuCommadIndex = 0;
	m_OutputCharSize = 0;
	m_SpeedRatio = speedRatio;

	//R}hXg쐬
	result = _MakeCommandList(pStr);
	if (result != 0) goto EXIT;

	//R}hȂȂ牽Ȃ
	if (m_KuchiPakuCommandList.size() == 0) {
		goto EXIT;
	}

	//UI^CvFbvVN
	result = m_pSysExOutCtrl->SendCSUIType(1);
	if (result != 0) goto EXIT;

	//{\[hFI
	result = m_pSysExOutCtrl->SendCSJPCharMode(1);
	if (result != 0) goto EXIT;

	//{ꕶ폜iׂāj
	result = m_pSysExOutCtrl->SendCSJPCharDel(0);
	if (result != 0) goto EXIT;

	//擪̃R}hs
	itr = m_KuchiPakuCommandList.begin();
	command = *itr;
	result = _ExecCommand(command);
	if (result != 0) goto EXIT;

EXIT:;
	return result;
}

//******************************************************************************
// R}hXg쐬
//******************************************************************************
int SEKuchiPakuCtrl::_MakeCommandList(const char* pStr)
{
	int result = 0;
	size_t pos = 0;
	SEKuchiPakuCommand command;
	char singleChar[4] = { '\0' };
	
	SingleCharType singleCharType;
	MouthShape mouthShape;
	unsigned char curMouthImage = SE_CS_FACE_MOUTH_CLOSE;

	//R}hXgNA
	m_KuchiPakuCommandList.clear();
	
	//
	while (true) {
		//R}h
		memset(&command, 0, sizeof(SEKuchiPakuCommand));
		command.mouthImage = SE_CS_FACE_MOUTH_CLOSE;
		command.intervalMSec = SE_KUCHIPAKU_INTERVAL_NORMAL_MSEC;
		command.displayString[0] = '\0';

		//1擾
		_GetSingleChar(&(pStr[pos]), singleChar);
		if (strlen(singleChar) == 0) {
			break;
		}
		strcat_s(command.displayString, 8, singleChar);
		pos += strlen(singleChar);
		_CheckSingleCharAttr(singleChar, &singleCharType, &mouthShape);
		
		//̌`XV
		curMouthImage = _GetMouthImage(mouthShape, curMouthImage);
		command.mouthImage = curMouthImage;
		
		//1ڂA/łȂꍇ
		if ((singleCharType != Seion) && (singleCharType != Dakuon)) {
			//R}ho^Ď
			if (singleCharType == Kutouten) {
				command.intervalMSec = SE_KUCHIPAKU_INTERVAL_KUTOUTEN_MSEC;
			}
			m_KuchiPakuCommandList.push_back(command);
			//̕
			continue;
		}

		//2ڂQƂ
		_GetSingleChar(&(pStr[pos]), singleChar);
		if (strlen(singleChar) == 0) {
			m_KuchiPakuCommandList.push_back(command);
			break;
		}
		_CheckSingleCharAttr(singleChar, &singleCharType, &mouthShape);

		//2ڂXijȂA
		if (singleCharType == Youon) {
			strcat_s(command.displayString, 8, singleChar);
			pos += strlen(singleChar);
			//Xŕꉹς邽ߌ̌`ύX
			curMouthImage = _GetMouthImage(mouthShape, curMouthImage);
			command.mouthImage = curMouthImage;
		}
		//2ڂijȂAăR}ho^Ď
		else if (singleCharType == Sokuon) {
			strcat_s(command.displayString, 8, singleChar);
			pos += strlen(singleChar);
			command.intervalMSec = SE_KUCHIPAKU_INTERVAL_SOKUON_MSEC;
			//̌`͕ύXȂ
			m_KuchiPakuCommandList.push_back(command);
			continue;
		}
		//2ڂi[jȂAăR}ho^Ď
		else if (singleCharType == Chouonpu) {
			strcat_s(command.displayString, 8, singleChar);
			pos += strlen(singleChar);
			command.intervalMSec = SE_KUCHIPAKU_INTERVAL_CHOUON_MSEC;
			//̌`͕ύXȂ
			m_KuchiPakuCommandList.push_back(command);
			continue;
		}
		//ȊO̕
		else {
			//AɃR}ho^Ď
			m_KuchiPakuCommandList.push_back(command);
			continue;
		}

		//3ڂQƂ
		_GetSingleChar(&(pStr[pos]), singleChar);
		if (strlen(singleChar) == 0) {
			m_KuchiPakuCommandList.push_back(command);
			break;
		}
		_CheckSingleCharAttr(singleChar, &singleCharType, &mouthShape);
		
		//3ڂijȂAăR}ho^
		if (singleCharType == Sokuon) {
			strcat_s(command.displayString, 8, singleChar);
			pos += strlen(singleChar);
			command.intervalMSec = SE_KUCHIPAKU_INTERVAL_SOKUON_MSEC;
			//̌`͕ύXȂ
			m_KuchiPakuCommandList.push_back(command);
		}
		//3ڂi[jȂAăR}ho^
		else if (singleCharType == Chouonpu) {
			strcat_s(command.displayString, 8, singleChar);
			pos += strlen(singleChar);
			command.intervalMSec = SE_KUCHIPAKU_INTERVAL_CHOUON_MSEC;
			//̌`͕ύXȂ
			m_KuchiPakuCommandList.push_back(command);
		}
		//ȊO̕
		else {
			//AɃR}ho^Ď
			m_KuchiPakuCommandList.push_back(command);
		}
	}

	//ŌɌR}hǉ
	command.mouthImage = SE_CS_FACE_MOUTH_CLOSE;
	command.intervalMSec = SE_KUCHIPAKU_INTERVAL_NORMAL_MSEC;
	command.displayString[0] = '\0';
	m_KuchiPakuCommandList.push_back(command);

//EXIT:;
	return result;
}

//******************************************************************************
// 1擾
//******************************************************************************
int SEKuchiPakuCtrl::_GetSingleChar(const char* pStr, char* singleChar)
{
	int result = 0;

	//1
	if (pStr[0] == '\0') {
		//I[
		singleChar[0] = '\0';
		goto EXIT;
	}
	else if (_IsSJISLeadByte(pStr[0]) && pStr[1] != '\0') {
		//2byte
		singleChar[0] = pStr[0];
		singleChar[1] = pStr[1];
		singleChar[2] = '\0';
	}
	else {
		//1byte
		singleChar[0] = pStr[0];
		singleChar[1] = '\0';
		goto EXIT;
	}

EXIT:;
	return result;
}

//******************************************************************************
// SHIFT-JIS 2oCg 擪1byte
//******************************************************************************
bool SEKuchiPakuCtrl::_IsSJISLeadByte(char c)
{
	bool isLeadByte = false;
	unsigned char uc = (unsigned char)c;

	if ((uc >= 0x81) && (uc <= 0x9F) || (uc >= 0xE0) && (uc <= 0xFC)) {
		isLeadByte = true;
	}

    return isLeadByte;
}

//******************************************************************************
// `FbN
//******************************************************************************
void SEKuchiPakuCtrl::_CheckSingleCharAttr(
		char* pSingleChar,
		SingleCharType* pSingleCharType,
		MouthShape* pMouthShape
	)
{
	//
	char seion_a[][4] = { "", "", "", "", "", "", "", "", "", "", "", "" };
	char seion_i[][4] = { "", "", "", "", "", "", "", "",             "", "" };
	char seion_u[][4] = { "", "", "", "", "", "", "", "", "",       "", "" };
	char seion_e[][4] = { "", "", "", "", "", "", "", "",             "", "" };
	char seion_o[][4] = { "", "", "", "", "", "", "", "", "", "", "", "" };
	char seion_n[][4] = { "", "" };
	// / 
	char dakuon_a[][4] = { "", "", "", "", "", "" };
	char dakuon_i[][4] = { "", "", "", "", "", "" };
	char dakuon_u[][4] = { "", "", "", "", "", "" };
	char dakuon_e[][4] = { "", "", "", "", "", "" };
	char dakuon_o[][4] = { "", "", "", "", "", "" };
	//X
	char youon_ya[][4] = { "", "" };
	char youon_yu[][4] = { "", "" };
	char youon_yo[][4] = { "", "" };
	//
	char sokuon_tsu[][4] = { "", "" };
	//
	char chouonpu[][4] = { "[", "" };
	//Ǔ_
	char kutouten[][4] = { "A", "B", "C", "D", "H", "I", "" };

	//
	if (_IsExistInList(seion_a[0], pSingleChar)) {
		*pSingleCharType = Seion;
		*pMouthShape = Mouth_A;
	}
	else if (_IsExistInList(seion_i[0], pSingleChar)) {
		*pSingleCharType = Seion;
		*pMouthShape = Mouth_I;
	}
	else if (_IsExistInList(seion_u[0], pSingleChar)) {
		*pSingleCharType = Seion;
		*pMouthShape = Mouth_U;
	}
	else if (_IsExistInList(seion_e[0], pSingleChar)) {
		*pSingleCharType = Seion;
		*pMouthShape = Mouth_E;
	}
	else if (_IsExistInList(seion_o[0], pSingleChar)) {
		*pSingleCharType = Seion;
		*pMouthShape = Mouth_O;
	}
	else if (_IsExistInList(seion_n[0], pSingleChar)) {
		*pSingleCharType = Seion;
		*pMouthShape = Mouth_Close;
	}
	// / 
	else if (_IsExistInList(dakuon_a[0], pSingleChar)) {
		*pSingleCharType = Dakuon;
		*pMouthShape = Mouth_A;
	}
	else if (_IsExistInList(dakuon_i[0], pSingleChar)) {
		*pSingleCharType = Dakuon;
		*pMouthShape = Mouth_I;
	}
	else if (_IsExistInList(dakuon_u[0], pSingleChar)) {
		*pSingleCharType = Dakuon;
		*pMouthShape = Mouth_U;
	}
	else if (_IsExistInList(dakuon_e[0], pSingleChar)) {
		*pSingleCharType = Dakuon;
		*pMouthShape = Mouth_E;
	}
	else if (_IsExistInList(dakuon_o[0], pSingleChar)) {
		*pSingleCharType = Dakuon;
		*pMouthShape = Mouth_O;
	}
	//X
	else if (_IsExistInList(youon_ya[0], pSingleChar)) {
		*pSingleCharType = Youon;
		*pMouthShape = Mouth_A;
	}
	else if (_IsExistInList(youon_yu[0], pSingleChar)) {
		*pSingleCharType = Youon;
		*pMouthShape = Mouth_U;
	}
	else if (_IsExistInList(youon_yo[0], pSingleChar)) {
		*pSingleCharType = Youon;
		*pMouthShape = Mouth_O;
	}
	//
	else if (_IsExistInList(sokuon_tsu[0], pSingleChar)) {
		*pSingleCharType = Sokuon;
		*pMouthShape = Mouth_Keep;
	}
	//
	else if (_IsExistInList(chouonpu[0], pSingleChar)) {
		*pSingleCharType = Chouonpu;
		*pMouthShape = Mouth_Keep;
	}
	//Ǔ_
	else if (_IsExistInList(kutouten[0], pSingleChar)) {
		*pSingleCharType = Kutouten;
		*pMouthShape = Mouth_Close;
	}
	//̑
	else {
		*pSingleCharType = Other;
		*pMouthShape = Mouth_Close;
	}

//EXIT:;
	return;
}

//******************************************************************************
// Xg݃`FbN
//******************************************************************************
bool SEKuchiPakuCtrl::_IsExistInList(
		char* pSingleCharList,
		char* pSingleChar
	)
{
	bool isContain = false;
	int i = 0;

	while (true) {
		if (strlen(&(pSingleCharList[i*4])) == 0) {
			//Xg
			break;
		}
		if (strcmp(&(pSingleCharList[i*4]), pSingleChar) == 0) {
			//Xgɑ݂Ă
			isContain = true;
			break;
		}
		i++;
	}

	return isContain;
}

//******************************************************************************
// 摜ʎ擾
//******************************************************************************
unsigned char SEKuchiPakuCtrl::_GetMouthImage(
	SEKuchiPakuCtrl::MouthShape mouthShape,
		unsigned char curMouthImage
	)
{
	unsigned char mouthImage = SE_CS_FACE_MOUTH_CLOSE;

	switch(mouthShape) {
		case Mouth_Close:
			mouthImage = SE_CS_FACE_MOUTH_CLOSE;
			break;
		case Mouth_A:
			mouthImage = SE_CS_FACE_MOUTH_A;
			break;
		case Mouth_I:
			mouthImage = SE_CS_FACE_MOUTH_I;
			break;
		case Mouth_U:
			mouthImage = SE_CS_FACE_MOUTH_U;
			break;
		case Mouth_E:
			mouthImage = SE_CS_FACE_MOUTH_E;
			break;
		case Mouth_O:
			mouthImage = SE_CS_FACE_MOUTH_O;
			break;
		case Mouth_MID:
			mouthImage = SE_CS_FACE_MOUTH_MID;
			break;
		case Mouth_Keep:
			mouthImage = curMouthImage;
			break;
		default:
			mouthImage = curMouthImage;
			break;
	}

	return mouthImage;
}

//******************************************************************************
// R}hs
//******************************************************************************
int SEKuchiPakuCtrl::_ExecCommand(SEKuchiPakuCommand command)
{
	int result = 0;
	unsigned char row = 0;
	UINT_PTR apiresult = 0;
	unsigned long intervalMSec = 0;

	//MTCY̍v32byte𒴂ꍇ͉iɒǉ
	if ((m_OutputCharSize + strlen(command.displayString)) > 32) {
		row = 1;
	}

	//SysExMFǉ
	if (strlen(command.displayString) > 0) {
		result = m_pSysExOutCtrl->SendCSJPCharAdd(row, &(command.displayString[0]));
		if (result != 0) goto EXIT;
		m_OutputCharSize += strlen(command.displayString);
	}

	//SysExMF̌`
	result = m_pSysExOutCtrl->SendCSFace(3, command.mouthImage);
	if (result != 0) goto EXIT;

	//ҋ@Ԋuɂςx𔽉f
	intervalMSec = (unsigned long)((float)(command.intervalMSec) / m_SpeedRatio);

	//^C}[Jn
	apiresult = SetTimer(
						m_hWnd,						//ʒmEBhE
						SE_TIMER_KUCHI_PAKU_NEXT,	//^C}[ID
						intervalMSec,				//^CAEgli~bj
						NULL						//^C}[֐
					);
	if (apiresult == 0) {
		result = YN_SET_ERR("Windows API error.", GetLastError(), 0);
		goto EXIT;
	}

	//R}hCfbNX
	m_KuchiPakuCommadIndex += 1;

EXIT:;
	return result;
}

//******************************************************************************
// ^C}[ʒm
//******************************************************************************
int SEKuchiPakuCtrl::OnTimer()
{
	int result = 0;
	SEKuchiPakuCommand command;
	SEKuchiPakuCommandList::iterator itr;

	//^C}[~
	KillTimer(m_hWnd, SE_TIMER_KUCHI_PAKU_NEXT);

	//R}h̑݊mF
	if (m_KuchiPakuCommadIndex >= m_KuchiPakuCommandList.size()) {
		//R}hȂ
		goto EXIT;
	}

	//R}hs
	itr = m_KuchiPakuCommandList.begin();
	advance(itr, m_KuchiPakuCommadIndex);
	command = *itr;
	result = _ExecCommand(command);
	if (result != 0) goto EXIT;

EXIT:;
	return result;
}


