//******************************************************************************
//
// SysEx2CS / SESysExOutCtrl
//
// SysExo͐NX
//
// Copyright (C) 2023 WADA Masashi. All Rights Reserved.
//
//******************************************************************************

#include "framework.h"
#include "SESysExOutCtrl.h"


//******************************************************************************
// RXgN^
//******************************************************************************
SESysExOutCtrl::SESysExOutCtrl(void)
{
	m_pOutDevCtrl = NULL;
	m_hLogListBox = NULL;
}

//******************************************************************************
// fXgN^
//******************************************************************************
SESysExOutCtrl::~SESysExOutCtrl(void)
{
}

//******************************************************************************
// 
//******************************************************************************
int SESysExOutCtrl::Initialize(
		SMOutDevCtrl* pOutDevCtrl,
		HWND hLogListBox
	)
{
	int result = 0;

	m_pOutDevCtrl = pOutDevCtrl;
	m_hLogListBox = hLogListBox;

//EXIT:;
	return result;
}

//******************************************************************************
// CureSynthZbg
//******************************************************************************
int SESysExOutCtrl::SendCSReset()
{
	int result = 0;
	unsigned char data[] = { 0xF0, 0x00, 0x48, 0x05, 0x7F, 0x00, 0x00, 0xF7 };

	//SysExM
	result = _SendSysEx(data, 8, "CureSynth Reset");
	if (result != 0) goto EXIT;
	
EXIT:;
	return result;
}

//******************************************************************************
// GM System ON
//******************************************************************************
int SESysExOutCtrl::SendGMSystemOn()
{
	int result = 0;
	unsigned char data[] = { 0xF0, 0x7E, 0x7F, 0x09, 0x01, 0xF7 };

	result = _SendSysEx(data, 6, "GM System ON");
	if (result != 0) goto EXIT;

EXIT:;
	return result;
}

//******************************************************************************
// GS Reset
//******************************************************************************
int SESysExOutCtrl::SendGSReset()
{
	int result = 0;
	unsigned char data[] = { 0xF0, 0x41, 0x10, 0x42, 0x12, 0x40, 0x00, 0x7F, 0x00, 0x41, 0xF7 };

	result = _SendSysEx(data, 11, "GS Reset");
	if (result != 0) goto EXIT;

EXIT:;
	return result;
}

//******************************************************************************
// XG System On
//******************************************************************************
int SESysExOutCtrl::SendXGSystemOn()
{
	int result = 0;
	unsigned char data[] = { 0xF0, 0x43, 0x10, 0x4C, 0x00, 0x00, 0x7E, 0x00, 0xF7 };

	result = _SendSysEx(data, 9, "XG System ON");
	if (result != 0) goto EXIT;

EXIT:;
	return result;
}

//******************************************************************************
// UI Type
//******************************************************************************
int SESysExOutCtrl::SendCSUIType(unsigned char type)
{
	int result = 0;
	char comment[256] = { '\0' };
	unsigned char data[] = { 0xF0, 0x00, 0x48, 0x05, 0x7F, 0x03, 0x00, 0xF7 };
	data[6] = type;

	sprintf_s(comment, 256, "UI Type Set (type=%u)", type);

	result = _SendSysEx(data, 8, comment);
	if (result != 0) goto EXIT;

EXIT:;
	return result;
}

//******************************************************************************
// 
//******************************************************************************
int SESysExOutCtrl::SendCSFace(unsigned char part, unsigned char image)
{
	int result = 0;
	char comment[256] = { '\0' };
	unsigned char data[] = { 0xF0, 0x00, 0x48, 0x05, 0x7F, 0x10, 0x00, 0x00, 0xF7 };
	data[6] = part;
	data[7] = image;

	sprintf_s(comment, 256, "UI Face set (part=%u, image=%u)", part, image);

	result = _SendSysEx(data, 9, comment);
	if (result != 0) goto EXIT;

EXIT:;
	return result;
}

//******************************************************************************
//  ܂΂
//******************************************************************************
int SESysExOutCtrl::SendCSFaceBlinking(unsigned char status)
{
	int result = 0;
	char comment[256] = { '\0' };
	unsigned char data[] = { 0xF0, 0x00, 0x48, 0x05, 0x7F, 0x04, 0x00, 0xF7 };
	data[6] = status;

	sprintf_s(comment, 256, "Face Blinking Set (status=%u)", status);

	result = _SendSysEx(data, 8, comment);
	if (result != 0) goto EXIT;

EXIT:;
	return result;
}

//******************************************************************************
// {\[h
//******************************************************************************
int SESysExOutCtrl::SendCSJPCharMode(unsigned char status)
{
	int result = 0;
	char comment[256] = { '\0' };
	unsigned char data[] = { 0xF0, 0x00, 0x48, 0x05, 0x7F, 0x05, 0x00, 0xF7 };
	data[6] = status;

	sprintf_s(comment, 256, "Japanese Character mode (status=%u)", status);

	result = _SendSysEx(data, 8, comment);
	if (result != 0) goto EXIT;

EXIT:;
	return result;
}

//******************************************************************************
// {ꕶǉ
//******************************************************************************
int SESysExOutCtrl::SendCSJPCharAdd(
		unsigned char row,
		char* pStr
	)
{
	int result = 0;
	int apiresult = 0;
	int i = 0;
	int size = 0;
	HWND hEdit = NULL;
	char comment_jpstr[256] = { '\0' };
	char comment[256] = { '\0' };
	unsigned char data[7 + (4 * 32) + 1];
	unsigned char head[] = { 0xF0, 0x00, 0x48, 0x05, 0x7F, 0x07, 0x00 };
	head[6] = row;
	unsigned char c = 0;

	memset(data, 0, (7 + (2 * 32) + 1));
	memcpy(data, head, 7);

	size = 7;
	for (i = 0; i < 32; i++) {
		if (pStr[i] == '\0') break;
		c = (unsigned char)pStr[i];
		data[size + 0] = (c & 0xF0) >> 4;
		data[size + 1] = (c & 0x0F);
		size += 2;
	}
	data[size] = 0xF7;
	size += 1;

	strcpy_s(comment_jpstr, 256, pStr);
	comment_jpstr[32] = '\0';
	sprintf_s(comment, 256, "Japanese Character Add (%s)", comment_jpstr);

	result = _SendSysEx(data, size, comment);
	if (result != 0) goto EXIT;

EXIT:;
	return result;
}

//******************************************************************************
// {ꕶ폜
//******************************************************************************
int SESysExOutCtrl::SendCSJPCharDel(unsigned char row)
{
	int result = 0;
	char comment[256] = { '\0' };
	unsigned char data[] = { 0xF0, 0x00, 0x48, 0x05, 0x7F, 0x06, 0x00, 0xF7 };
	data[6] = row;

	sprintf_s(comment, 256, "Japanese Character Delete (row=%u)", row);

	result = _SendSysEx(data, 8, comment);
	if (result != 0) goto EXIT;

EXIT:;
	return result;
}

//******************************************************************************
// SysExM
//******************************************************************************
int SESysExOutCtrl::_SendSysEx(
		unsigned char* pData,
		unsigned long size,
		const char* pComment
	)
{
	int result = 0;
	char hexStr[16];
	unsigned long i = 0;
	std::string str;

	//SysExM
	result = m_pOutDevCtrl->SendLongMsg(0, pData, size);
	if (result != 0) goto EXIT;

	//Mf[^16iɕϊ
	for (i = 0; i < size; i++) {
		sprintf_s(hexStr, 16, "%02X ", pData[i]);
		str += hexStr;
	}

	//Rgǉ
	str += "# ";
	str += pComment;

	//Oo
	result = _OutputLog(str.c_str());
	if (result != 0) goto EXIT;

EXIT:;
	return result;
}

//******************************************************************************
// Oo
//******************************************************************************
int SESysExOutCtrl::_OutputLog(const char* pStr)
{
	int result = 0;
	LRESULT lresult = 0;
	size_t listCount = 0;

	//TODO:Oo͏͂ɎȂ

	//Xgs擾
	lresult = SendMessage(m_hLogListBox, LB_GETCOUNT, 0, 0);
	if (lresult == LB_ERR) {
		result = YN_SET_ERR("Windows API error.", GetLastError(), (DWORD64)lresult);
		goto EXIT;
	}
	listCount = lresult;

	//XgsɒBĂꍇ̓Xg擪1s폜
	if (listCount >= SE_LOG_COUNT_MAX) {
		lresult = SendMessage(m_hLogListBox, LB_DELETESTRING, 0, 0);
		if (lresult == LB_ERR) {
			result = YN_SET_ERR("Windows API error.", GetLastError(), (DWORD64)lresult);
			goto EXIT;
		}
		listCount--;
	}

	//XgɃO1sǉ
	lresult = SendMessage(m_hLogListBox, LB_ADDSTRING, 0, (LPARAM)pStr);
	if ((lresult == LB_ERR) || (lresult == LB_ERRSPACE)) {
		result = YN_SET_ERR("Windows API error.", GetLastError(), (DWORD64)lresult);
		goto EXIT;
	}
	listCount++;

	//XgIăXN[\
	lresult = SendMessage(m_hLogListBox, LB_SETCURSEL, listCount - 1, 0);
	if (lresult == LB_ERR) {
		result = YN_SET_ERR("Windows API error.", GetLastError(), (DWORD64)listCount);
		goto EXIT;
	}

EXIT:;
	return result;
}


